from paste.script import templates

class OortAppTemplate(templates.Template):
    summary = "A clean Oort web app package"
    _template_dir = 'paste_templates/oort_app'
    required_templates = ['basic_package']
    use_cheetah = False




#=======================================================================
import os
from os.path import splitext, expanduser
from rdflib import Namespace
#=======================================================================
# TODO: (optional?) poorMansInference!
#           - interpret rdfs (also using equiv. owl terms)
#           - optional vocabulary discovery?


OUG_NS = Namespace('tag:oort.to,2006:system/util/graphs#')
LAST_MOD = OUG_NS.lastmodified

def load_if_modified(graph, fpath, format):
    dataUri = graph.absolutize(fpath)
    modTime = os.stat(fpath).st_mtime
    try:
        loadedModValue = graph.value(dataUri, LAST_MOD)
        loadedModTime = int(loadedModValue)
    except TypeError:
        loadedModTime = -1
    if modTime > loadedModTime:
        graph.remove((dataUri, LAST_MOD, loadedModValue))
        graph.remove_context(graph.context_id(dataUri))
        print "Loading: %s" % dataUri
        graph.load(fpath, publicID=dataUri, format=format)
        graph.add((dataUri, LAST_MOD, modTime))


DEFAULT_FORMAT_MAP = {
        # TODO: rdf/a
        'rdf': 'xml',
        'n3':'n3',
    }

def get_format(fpath, fmap=None):
    fmap = fmap or DEFAULT_FORMAT_MAP
    ext = splitext(fpath)[-1][1:]
    return fmap.get(ext)


def loader(graph, base=None, formatMap=None):
    if base: base = expanduser(base) + '%s'
    def load_data(fpath):
        if base:
            fpath = base % fpath
        load_if_modified(graph, fpath, format=get_format(fpath, formatMap))
    return load_data


def load_dir(graph, basedir, formatMap=None, errorHandler=None):
    basedir = expanduser(basedir)
    for base, fdirs, fnames in os.walk(basedir):
        for fname in fnames:
            fpath = os.path.join(base, fname)
            format = get_format(fpath, formatMap)
            if not format:
                continue
            try:
                load_if_modified(graph, fpath, format)
            except Exception, e:
                if errorHandler:
                    errorHandler(e, fpath)
                else:
                    raise e





# -*- coding: UTF-8 -*-
#--------------------------------------------------------------------------------
# The Oort BSD-style License
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of Oort nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#--------------------------------------------------------------------------------
"""
========================================================================
Oort
========================================================================

Oort is a toolkit for creating RDF_-driven WSGI_-compliant web applications.

The purpose of this is to make it easy to create web views of RDF Graphs by 
using some declarative python programming.

Oort uses RDFLib_, Paste_ and Genshi_ for the heaving lifting. (Initial but 
rudimentary support for `Template Plugins`_ alá Buffet and TurboGears is also 
included.)

.. _RDF: http://www.rdfabout.net
.. _WSGI: http://wsgi.org
.. _RDFLib: http://rdflib.net
.. _Paste: http://pythonpaste.org
.. _Genshi: http://genshi.edgewall.org/
.. _`Template Plugins`: http://www.turbogears.org/docs/plugins/template.html

Overview
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

These are the main packages:

``oort.sitebase``
    Contains classes used for declarative definitions of *displays*, used for 
    matching resources and rendering a particular output (html, json etc.). By 
    defining *aspects*, the type (or super-type) of a selected resource is 
    mapped to a particular RdfQuery and an associated template.

    One or more displays are put in the context of a *resource viewer*, which 
    becomes a WSGI application ready to mount in your WSGI environment.

``oort.rdfview``
    Contains classes and functions used for defining RDF queries and selectors, 
    i.e. declarations used to pick properties and associated sub-queries from a
    chosen resource (similar to how many ORM-toolkits work).

How?
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A couple of RdfQuerys::

    from oort.rdfview import *
    from myapp.ns import SITE # your own..

    class Node(RdfQuery):
        title = localized(DC)

    class SiteNode(Node):
        altTitle = localized(DC.alternative)
        relations = collection(SITE) >> Node
        nodeContent = localized_xml(SITE)

    class Person(RdfQuery):
        name = one(FOAF)
        knows = each(FOAF) >> THIS_QUERY

And a web application::

    from oort.sitebase import *
    from myapp import queries
    from myapp.ns import SITE

    class ExampleViewer(ResourceViewer):

        resourceBase = "http://example.org/oort/"
        langOrder = 'en', 'sv'

        class PlainWebDisplay(Display):
            name = "main"
            default = True
            outputMethod = 'xhtml'
            outputEncoding  = 'iso-8859-1'
            templateBase = "view/mainweb"
            globalQueries = {'languages': queries.sitelabels }
            aspects = [
                    Aspect(SITE.SiteNode, "sitenode.xhtml",
                            {'node': queries.SiteNode}) ,
                    Aspect(FOAF.Person, "person.xhtml",
                            {'person': queries.Person}) ,
                    Aspect(RDFS.Resource, "not_found.xhtml")
                ]

        class JsonDisplay(Display):
            name = "json"
            contentType = 'application/x-javascript'
            aspects = [
                    JsonAspect(SITE.SiteNode, {'node': queries.SiteNode})
                ]

But wait, there's more..

Paste!
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Makes WSGI go down smoothly. ResourceViewers take RDFLib ``Graph`` instances in 
their constructors and become callables adhering to the spec.

To get started quickly, run::

    $ paster create -t oort_app
    ... fill in desired values in the dialogue
    $ cd myapp/
    $ vim # edit and test..
    $ chmod u+x webapp.ini
    $ ./webapp.ini

Why?
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Because RDF is a formidable technology that could revolutionize the way 
information is treated and shared. Python and WSGI are exemplary technologies 
to use when building applications dealing with such data.

Why *Oort*?
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

* The Oort Cloud is a fascinating, alien phenomenon in a region *less 
  travelled*..
* Imagine all the BNodes in that cloud just waiting for an URI on the Web..
* "Output-Oriented RDF Toolkit"!
* a Dutch surname, so the way may be obvious to Oort..

--------------------------------------------------------------------------------

Copyright (c) 2006, 2007 Niklas Lindström <lindstream@gmail.com>.
All rights reserved.

License: BSD-style <http://opensource.org/licenses/bsd-license>.

"""

__docformat__ = 'reStructuredText en'
__author__ = "Niklas Lindström"
__version__ = "0.3.2"




# TODO: rewrite using only elementtree..(?)
# TODO: cannot handle ordered fallbacks (will filter all others or none)
#        - solutions may have to expand all literals, store their language and
#          fallback accordingly, or use a rather complex simultaneous iteraror
#          loop on all values.

from genshi import XML
from genshi.core import Stream

START, END = Stream.START, Stream.END
XML_LANG = u'{http://www.w3.org/XML/1998/namespace}lang'
RDF_WRAPPER = u'rdf-wrapper'

def language_filtered_xml(valueOrList, lang, fragment=True, encoding=None):
    if isinstance(valueOrList, unicode):
        return langXML(valueOrList, lang, fragment, encoding)
    else:
        # TODO: use flattened iterator instead..(?)
        events = []
        for value in valueOrList:
            if value:
                events.extend( langXML(value, lang, fragment, encoding).events )
        return Stream(events)

def langXML(text, lang, fragment=True, encoding=None):
    if text.startswith('<?xml ') or text.startswith('<!DOCTYPE '):
        fragment = False
    if fragment:
        text = '<xml>%s</xml>' % text
    if isinstance(text, unicode):
        encoding = 'utf-16'
        text = text.encode(encoding)
    stream = XML(text)
    lang_filter = filter_language(lang)
    if fragment:
        return stream | skip_outer | lang_filter
    else:
        return stream | lang_filter

def skip_outer(stream):
    """A filter that doesn't actually do anything with the stream."""
    istream = iter(stream)
    istream.next()
    last = None
    for content in istream:
        if last: yield last
        last = content

def filter_language(lang):
    def filter_lang(stream):
        depth = 0
        eating = False
        for kind, data, pos in stream:
            if kind == START and lang:
                elemLang = get_elem_lang(data)
                if elemLang:
                    # TODO: remove? remove_lang_attrib(data)
                    if elemLang != lang:
                        eating = True
            if eating == True:
                if kind == START:
                    depth += 1
                elif kind == END:
                    depth -= 1
                    if depth == 0: eating = False
                continue

            if kind == START and data[0] == RDF_WRAPPER:
                continue
            elif kind == END and data == RDF_WRAPPER:
                continue
            else:
                yield kind, data, pos

    return filter_lang

def get_elem_lang(data):
    # FIXME: Remove 'lang', it's a temporary solution due
    # to rdflib xml/rdf XMLLiteral with embedded xml:lang
    # parsing problems..
    for name, value in data[1]:
        if name in (XML_LANG, 'lang'):
            return value





# Based on design in:
#   <http://www.turbogears.org/docs/plugins/template.html>
#   <http://projects.dowski.com/view/buffet>

import pkg_resources

available_engines = dict([ (entry_point.name, entry_point.load())
        for entry_point in pkg_resources.
                iter_entry_points('python.templating.engines') ])

class TemplateEngineManager(object):

    def __init__(self, extra_vars_func=None, engine_opts=None):
        self.extra_vars_func = extra_vars_func
        self.engine_opts = engine_opts
        self.engines = {}

    def get_engine_and_path(self, path, default_engine=None):
        if not ':' in path:
            dotted_path = path
            engine_name = default_engine
        else:
            engine_name, dotted_path = path.split(':', 1)
        engine = self._get_engine(engine_name)
        return engine, dotted_path

    def render_template(self, path, data, format, fragment=False):
        engine, dotted_path = self.get_engine_and_path(path)
        result = engine.render(
                data, format=format, fragment=fragment, template=dotted_path)
        return result

    def _get_engine(self, engine_name):
        engine = self.engines.get(engine_name)
        if not engine:
            Engine = available_engines.get(engine_name, None)
            if not Engine:
                msg = 'No engine for "%s" found. Please install it.'
                raise TemplateEngineMissing(msg % (engine_name,))
            self.engines[engine_name] = \
                    engine = Engine(self.extra_vars_func, self.engine_opts)
        return engine

class TemplateEngineMissing(Exception):
    pass




from rdflib import ConjunctiveGraph, URIRef
from oort.rdfview import *
from oort.test import mergepaths

# TODO: test some with rdflib.RDFS to make sure module namespaces work.

T = Namespace("http://example.org/oort/test#")

class Part(RdfQuery):
    name = one(T)

class Item(Part):
    title = localized(T)
    relations = each(T.relation) >> THIS_QUERY
    unaryRelation = one(T) >> Part
    collection = collection(T) >> Part
    xmlData = localized_xml(T)
    titleLang = i18n_dict(T['title'])
    labels = each_localized(T.label)


def assert_contains(graph, query):
    for spo in query.to_graph():
        assert spo in graph


#=======================================


TEST_DATA = mergepaths(__file__, 'testdata.n3')
graph = ConjunctiveGraph()
graph.load(open(TEST_DATA), format='n3')

itemX = URIRef('tag:oort.to,2006:test:item:x')
en, sv = 'en', 'sv'

def test_one():
    assert Item(graph, en, itemX).name == u'Item X'

def test_one_relation():
    assert Item(graph, en, itemX).unaryRelation.name == u'One Relation'

def test_each():
    relItemNames = [item.name for item in Item(graph, en, itemX).relations]
    assert set(relItemNames) == set(["Related 1", "Related 2"])

def test_collection():
    collItems = [item.name for item in Item(graph, en, itemX).collection]
    assert set(collItems) == set(["Part 1", "Part 2"])

def assert_equals(v1, v2): assert v1 == v2

def test_localized():
    for l, v in [(en, u'Example Item'), (sv, u'Exempelsak')]:
        yield assert_equals, Item(graph, l, itemX).title, v

class RichItem(Item):
    xmlData = localized_xml(T)

def test_localized_xml():
    for l, v in [(en, u'XML Data'), (sv, u'XML-data')]:
        treeData = list(RichItem(graph, l, itemX).xmlData)
        ok = False
        for stream in treeData:
            for item in stream:
                if v in item: ok = True
        assert ok

def test_i18n_dict():
    titleLangs = Item(graph, en, itemX).titleLang
    assert set(titleLangs.keys()) == set(['en', 'sv'])
    assert set(titleLangs.values()) == set([u'Example Item', u'Exempelsak'])

def test_each_localized():
    assert set(Item(graph, en, itemX).labels) == set([u'En', u'eN'])
    assert set(Item(graph, sv, itemX).labels) == set([u'Sv', u'sV'])


class Creation(RdfQuery):
    createdBy = one_where_self_is(T.creation) >> Part

def test_one_where_self_is():
    creation = Creation(graph, en, itemX)
    assert creation.createdBy.name == "Creator"
    assert_contains(graph, creation)


class Owner(RdfQuery):
    owns = each_where_self_is(T.owner) >> Part

def test_each_where_self_is():
    owner = Owner(graph, en, itemX)
    owned = [part.name for part in owner.owns]
    assert set(owned) == set(["Part A", "Part B"])
    assert_contains(graph, owner)


def test_to_json():
    json = Item(graph, en, itemX).to_json()
    d = eval(json)
    # TODO: inspect!


def test_to_graph():
    itemGraph = Item(graph, en, itemX).to_graph()
    # TODO: inspect!


class CustomItem(Item):
    @selector
    def randomRelation(self, graph, lang, subject):
        from random import randint
        assert isinstance(self, CustomItem)
        itemUris = [uri for uri in graph.subjects(RDF.type, T.Item)
                        if uri != subject]
        return Part(graph, lang, itemUris[randint(0, len(itemUris)-1)])

def test_selector_decorator():
    assert CustomItem(graph, en, itemX).randomRelation.name.startswith("Related ")


class FilteredItem(RdfQuery):
    sortedKeywords = each(T.keyword) | Sorter()
    sortedKeywordsReversed = each(T.keyword) | Sorter(reverse=True)
    someKeywords = each(T.keyword) | Filter(lambda v: v in ('q', 'r', 'y'))
    name_upper = one(T.name) | (lambda v: unicode(v).upper())

def test_filter():
    item = FilteredItem(graph, en, itemX)

    yield assert_equals, item.name_upper, "ITEM X"

    def assert_sorted(item):
        keywords = "e q r t w y".split()
        assert item.sortedKeywords == keywords
        keywords.reverse()
        assert item.sortedKeywordsReversed == keywords

    yield assert_sorted, item

    def assert_some(item):
        assert set(item.someKeywords) == set(['q', 'r', 'y'])

    yield assert_some, item


class ImplicitItem(RdfQuery):
    _rdfbase_ = T
    name = one()

def test_implicit_item():
    item = ImplicitItem(graph, en, itemX)
    assert item.name == "Item X"
    assert ImplicitItem.RDF_TYPE == T.ImplicitItem

class TypedImplicitItem(RdfQuery):
    _rdfbase_ = T
    RDF_TYPE = T.OtherType
    name = one()

def test_typed_implicit_item():
    item = TypedImplicitItem(graph, en, itemX)
    assert item.name == "Item X"
    assert TypedImplicitItem._rdfbase_ == T
    assert TypedImplicitItem.RDF_TYPE == T.OtherType


def test_from_dict():
    data = dict(
        name = "Item X",
        title = "Example Item",
        titleLang = { 'en': "Example Item", 'sv': "Exempelsak" },
        labels = [ "En", "eN" ],
        relations = [
                {'name': "Related 1"},
                {'name': "Related 2"},
            ],
        xmlData = "<div/>",
    )
    item = Item.from_dict(data, "en", URIRef("tag:oort.to,2006/test/item/x_fromdict"))
    # TODO: verify graph!
    item.to_graph()
    #print item.to_rdf(); raise Exception


def test_bound_with():
    bound = Item.bound_with(itemX)
    yield assert_equals, bound.query, Item
    yield assert_equals, bound.__name__, 'Item'
    yield assert_equals, bound(graph, en, BNode()).name,  u'Item X'
    bound = Item.bound_with(itemX, sv)
    yield assert_equals, bound(graph, en, BNode()).title, u'Exempelsak'





#=======================================================================
from rdflib import ConjunctiveGraph, Namespace, URIRef, RDF, RDFS
from oort.display import AspectBase, TemplateAspectBase
from oort.display import SubTypeAwareDisplay
#=======================================================================


def test_AspectBase(cfg={}, maker=None):
    dummyType = object()
    queries = {'a': 0}
    if maker:
        aspect = maker(dummyType, queries)
    else:
        class TestAspect(AspectBase):
            def post_init_configure(self, cfg):
                self.cfg = cfg
        aspect = TestAspect(dummyType, queries)
    globalQueries = {'a': 1, 'b': 2}
    assert aspect.forType == dummyType
    assert aspect.queries == queries

    aspect.post_init_setup(globalQueries, cfg)
    if not maker:
        assert aspect.cfg == cfg

    assert aspect.queries == {'a': 0, 'b': 2}
    return aspect

def test_TemplateAspectBase():
    tbase = "TEMPLATE_BASE"
    tname = "TEMPLATE"
    cfg = {'templateBase': tbase}
    def maker(forType, queries):
        return TemplateAspectBase(forType, tname, queries)
    aspect = test_AspectBase(cfg, maker)
    assert aspect.templateBase == tbase
    assert aspect.templateName == tname


ont = Namespace("urn:ont#")

class TestSubTypeAwareDisplay:

    def setup(self):
        typeAspects = {ont.T1: "T1 handler", ont.T5: "T5 handler"}
        #typeAspects = {ont.T2: "handler for T2", ont.T3: "handler for T3"}
        self.display = SubTypeAwareDisplay()
        self.display.typeAspects = typeAspects

    def test_create_match_graph(self):
        # TODO: verify test accuracy; then remake into test generator (see nose)

        graph = ConjunctiveGraph()

        def classAndSubClass(C1, C2):
            graph.add( (C1, RDF.type, RDFS.Class) )
            graph.add( (C2, RDF.type, RDFS.Class) )
            graph.add( (C2, RDFS.subClassOf, C1) )

        classAndSubClass(ont.T1, ont.T2)
        classAndSubClass(ont.T2, ont.T3)
        classAndSubClass(ont.T3, ont.T4)
        classAndSubClass(ont.T4, ont.T5)

        item1 = URIRef("urn:item1")
        item2 = URIRef("urn:item2")
        item3 = URIRef("urn:item3")
        item4 = URIRef("urn:item4")
        item5 = URIRef("urn:item5")

        graph.add( (item1, RDF.type, ont.T1) )
        graph.add( (item2, RDF.type, ont.T2) )
        graph.add( (item3, RDF.type, ont.T3) )
        graph.add( (item4, RDF.type, ont.T4) )
        graph.add( (item5, RDF.type, ont.T5) )

        matchGraph = self.display.create_match_graph(graph)

        def get_aspect(resource):
            for rdfType in matchGraph.objects(resource, RDF.type):
                return self.display.typeAspects.get(rdfType)

        assert get_aspect(item1) == "T1 handler"
        assert get_aspect(item2) == "T1 handler"
        assert get_aspect(item3) == "T1 handler"
        assert get_aspect(item4) == "T1 handler"
        assert get_aspect(item5) == "T5 handler"





#=======================================================================
from rdflib import ConjunctiveGraph
from oort.sitebase import MultiBaseResourceViewer, AspectBase
#=======================================================================


class TestResourceViewerBase:
    pass # TODO: test at least __call__- and app_factory!


def test_MultiBaseResourceViewer():
    # TODO: verify test accuracy; then remake into test generator (see nose)

    class TestViewer(MultiBaseResourceViewer):
        resourceBases = {
            'ont': ("http://example.org/rdfns/ontology#", ''),
            'ex': ("http://example.org/resources/", "/"),
            'urn': ("urn:example-org:", ":"),
        }
        defaultResource = 'http://example.org/default/resource'

    viewer = TestViewer(ConjunctiveGraph())
    def test_path(path, expected, samePath=True):
        resource = viewer.resource_from(path.split('/'))
        assert resource == expected
        path2, _success = viewer.resource_to_app_path(resource)
        assert (path == path2) is samePath

    test_path("ex/path/to/resource", "http://example.org/resources/path/to/resource")
    test_path("ex/", "http://example.org/resources/")
    test_path("ont/OneConcept", "http://example.org/rdfns/ontology#OneConcept")
    test_path("ont/someRelationTo", "http://example.org/rdfns/ontology#someRelationTo")
    test_path("bad/nothing/to/see/here", "http://example.org/default/resource", False)
    test_path("bad/", "http://example.org/default/resource", False)
    test_path("/", "http://example.org/default/resource", False)
    test_path("", "http://example.org/default/resource", False)


def test_using():
    dummy = object()
    aspect = AspectBase(None)
    assert aspect == aspect.using(item=dummy)
    assert aspect.queries['item'] == dummy





# -*- coding: UTF-8 -*-
"""
"""
#=======================================================================
import os.path
from rdflib import RDF, RDFS, ConjunctiveGraph
from genshi.output import DocType
from genshi.template import TemplateLoader
from oort.util import templating
from oort.util.code import contract, autosuper
__metaclass__ = autosuper
#=======================================================================
# TODO: Remove all variant handling; too much arbitrariness.
# If a need for aspect-switching on request details arise, re-invent.
# (This is reasonably done within the template system).
# TODO: use Buffet-style templating instead(?)


class DisplayBase:

    contentType = None

    @contract.template_method
    def get_result_generator(self, graph, appUtil, variantArg):
        raise NotImplementedError


class Display(DisplayBase):

    # TODO: for outputEncoding, see Aspect problems below.
    #       May need to redesign to set template-speficic stuff instead.
    #       These options should be available as data too (e.g. for template
    #       output and/or encoding specific stuff..).
    outputMethod = None
    outputEncoding  = None

    templateBase = ""
    globalQueries = None
    aspects = []

    def __init__(self, cfg=None):
        self.typeAspects = {}
        self.variantAspects = {}
        cfg = cfg or {}
        cfg['templateBase'] = self.templateBase
        for aspect in self.aspects:
            aspect.post_init_setup(self.globalQueries, cfg=cfg)
            if aspect.forVariant:
                variant = self.variantAspects.setdefault(aspect.forVariant, {})
                typeDict = variant
            else:
                typeDict = self.typeAspects
            typeDict[aspect.forType] = aspect

    @contract.default_method
    def parse_variant(self, variantStr):
        return variantStr

    def get_result_generator(self, graph, appUtil, variantArg):
        variant = self.parse_variant(variantArg)
        data = self.get_extra_data(appUtil, variant) or {}
        data['app'] = appUtil
        resource, lang = appUtil.current.resource, appUtil.current.lang
        aspect = self.get_aspect(graph, resource, variant)
        generate = aspect.make_result_generator(graph, resource, lang, data)
        return generate(self.outputMethod, encoding=self.outputEncoding)

    def get_aspect(self, graph, resource, variant):
        rdfType = RDFS.Resource
        for _type in graph.objects(resource, RDF.type):
            rdfType = _type; break
        typeDict = self.typeAspects
        if variant:
            typeDict = self.variantAspects.get(variant) or self.typeAspects
        return typeDict.get(rdfType) or typeDict[RDFS.Resource]

    @contract.default_method
    def get_extra_data(self, appUtil, variant):
        "Use for further template preparation based on current request."
        return None


class KeywordVariantParser:
    pass # TODO


class SubTypeAwareDisplay(Display):

    def __init__(self, cfg=None):
        self.__super.__init__(cfg)
        self._matchGraph = None

    def get_aspect(self, graph, resource, variant):
        if not self._matchGraph:
            self._matchGraph = self.create_match_graph(graph)
        return self.__super.get_aspect(self._matchGraph, resource, variant)

    def create_match_graph(self, graph):
        """For optimization, creating a sparse graph of "best matching"
        for all resources and querying only that at runtime. """
        typeAspects = self.typeAspects
        def list_with_sub_types(rdfType):
            yield rdfType
            for subType in graph.subjects(RDFS.subClassOf, rdfType):
                if subType not in typeAspects:
                    for subSub in list_with_sub_types(subType):
                        yield subSub

        sparseGraph = ConjunctiveGraph()
        for handledType in typeAspects:
            for anySub in list_with_sub_types(handledType):
                for resource in graph.subjects(RDF.type, anySub):
                    sparseGraph.add((resource, RDF.type, handledType))

        return sparseGraph

    # TODO: Implement? Not really useful if create_match_graph is used;
    # however, this one is *much* better if resources are volatile..
    #def get_aspect(resource):
    #    for rType in graph.objects(resource, RDF.type):
    #        if rType in _mostSpecificHandled:
    #            return typeAspects.get(rType)
    #        supType = _handledSupers.get(rType)
    #        if supType:
    #            return typeAspects.get(supType)
    #    return typeDict[RDFS.Resource]


class AspectBase:

    def __init__(self, forType, queries=None, forVariant=None):
        self.forType = forType
        self.queries = queries or {}
        self.forVariant = forVariant

    @contract.state_change
    def post_init_setup(self, globalQueries, cfg=None):
        if globalQueries:
            queries = globalQueries.copy()
            queries.update(self.queries)
            self.queries = queries
        self.post_init_configure(cfg or {})

    @contract.template_method
    def post_init_configure(self, cfg):
        pass

    @contract.helper
    def generate_query_items(self, graph, resource, lang):
        """Generates key, value pairs which constitute named, run queries."""
        if self.queries:
            for name, query in self.queries.items():
                yield name, query(graph, lang, resource)

    # TODO: redesign to set (default) encoding and output(method) via cfg in Aspect#post_init_setup?
    @contract.template_method
    def make_result_generator(self, graph, resource, lang, data=None):
        """Returns a generator that takes the arguments 'encoding' and
        'output'."""
        raise NotImplementedError

    def using(self, **queries):
        self.queries = queries
        return self

class TemplateAspectBase(AspectBase):
    def __init__(self, forType, templateName, queries=None, forVariant=None):
        self.__super.__init__(forType, queries, forVariant)
        self.templateName = templateName
        self.templateBase = None

    def post_init_configure(self, cfg):
        self.templateBase = cfg.get('templateBase')

class Aspect(TemplateAspectBase):

    RELOAD_TEMPLATES = "reload_templates"

    # TODO: use coming genshi.template.plugin.MarkupTemplateEnginePlugin.doctypes
    doctypes = dict(
            (key.lower().replace('_', '-'), item)
            for key, item in DocType.__dict__.items()
            if key.isupper() and isinstance(item, tuple)
        )

    def post_init_configure(self, cfg):
        self.__super.post_init_configure(cfg)
        autoReload = str(cfg.get(self.RELOAD_TEMPLATES, False)) == str(True)
        self.loader = TemplateLoader(['.'], auto_reload=autoReload)

    def load_template(self):
        return self.loader.load(os.path.join(self.templateBase, self.templateName))

    def make_result_generator(self, graph, resource, lang, data=None):
        tplt = self.load_template()
        allData = dict( self.generate_query_items(graph, resource, lang) )
        if data: allData.update(data)
        # TODO: although genshi has a serialize (to stream) too, it doesn't
        # take encoding. Find the simplest way to get around that (without too
        # much duplicatiom of render innards..).. Or just remove this todo?
        def generator(format, encoding):
            doctype = self.doctypes.get(format)
            format = format.split('-')[0]
            yield tplt.generate(**allData).render(
                    format, doctype=doctype, encoding=encoding)
        return generator


class TpltAspect(TemplateAspectBase):

    engineManager = templating.TemplateEngineManager(None, {})

    defaultEngine = "genshi"

    def post_init_configure(self, cfg):
        self.__super.post_init_configure(cfg)
        self.engine, self.path = self.engineManager.get_engine_and_path(
                self.templateBase+'.'+self.templateName, self.defaultEngine)
        template = self.engine.load_template(self.path)
        self.template = template or None

    def make_result_generator(self, graph, resource, lang, data=None):
        allData = dict(self.generate_query_items(graph, resource, lang))
        if data:
            allData.update(data)
        engine = self.engine
        template = self.template or self.path
        # FIXME: needs to use encoding, but alas, the plugin format precludes it...
        def generator(format, encoding):
            result = engine.render(allData, format=format, template=template)
            yield result
        return generator


class RdfAspect(AspectBase):

    def __init__(self, forType, queries=None):
        self.__super.__init__(forType, queries)

    def make_result_generator(self, graph, resource, lang, data=None):
        qData = dict( self.generate_query_items(graph, resource, lang) )
        def generator(format, encoding):
            lgraph = ConjunctiveGraph()
            for key, result in qData.items():
                lgraph += result.to_graph()
            yield lgraph.serialize(format='pretty-xml')
        return generator

class JsonAspect(AspectBase):

    def __init__(self, forType, queries=None, subjectKey=False):
        self.__super.__init__(forType, queries)
        self.subjectKey = subjectKey

    def make_result_generator(self, graph, resource, lang, data=None):
        qData = dict( self.generate_query_items(graph, resource, lang) )
        def generator(format, encoding):
            # TODO: use encoding!
            yield '{\n'
            for i, (key, result) in enumerate(qData.items()):
                if i != 0: yield ","
                yield "'%s':" % key
                # TODO: a very crude way of dealing with long text..
                yield result.to_json(self.subjectKey).replace('}', '}\n')
            yield '}\n'
        return generator





#=======================================================================


class autosuper(type):
    def __init__(cls, name, bases, dict):
        super(autosuper, cls).__init__(name, bases, dict)
        setattr(cls, "_%s__super" % name, super(cls))


#=======================================================================


def init_slots(obj, *args, **kwargs):
    data = dict(zip(obj.__slots__, args))
    data.update(kwargs)
    for a in obj.__slots__:
        setattr(obj, a, data.get(a, None))

class SlotStruct(object):
    __slots__ = ()
    def __init__(self, *args, **kwargs):
        init_slots(self, *args, **kwargs)
    def __str__(self):
        return "%s instance with %s" % (type(self).__name__, str(self.make_dict()))
    def make_dict(self):
        return dict([(a, getattr(self, a)) for a in self.__slots__])


#=======================================================================


def property_with_setter(fset):
    attr = '_%s' % fset.func_name
    return property(lambda self: getattr(self, attr),
            fset, doc=fset.__doc__)


#=======================================================================


def call(caller, *args):
    def decorator(decorated):
        callArgs = [(arg, decorated)[arg is call] for arg in args]
        caller(*callArgs)
        return call
    return decorator


#=======================================================================


class BooleanDecorator(object):
    """
    A documentation helper. Usage:

        >>> contract = BooleanDecorator('interface')
        >>> class T:
        ...     @contract.interface
        ...     def action(self): raise NotImplementedError
        ...
        >>> assert T.action.interface == True
    """

    __slots__ = ('attributes')

    def __init__(self, *args):
        self.attributes = args

    def __getattr__(self, name):
        if name not in self.attributes:
            raise Exception("Unknown attribute: %s" % name)
        def decorator(func):
            setattr(func, name, True)
            return func
        return decorator

contract = BooleanDecorator('template_method', 'default_method', 'helper', 'state_change')


def attrs(**kwargs):
    """
    Generic decorator for setting attributes. Usage:

        >>> @attrs(value="A value", thing=None, other=True)
        ... def f(): pass
        ...
        >>> assert f.value == "A value"
        >>> assert f.thing is None
        >>> assert f.other is True
    """
    def decorator(func):
        # TODO: or just func.__dict__.update(kwargs)
        for key, value in kwargs.items():
            setattr(func, key, value)
        return func
    return decorator





from rdflib import URIRef, RDF, RDFS
from oort.rdfview import RdfQuery, one, localized, i18n_dict
from oort.util.code import SlotStruct

class HasLabel(RdfQuery):
    label = localized(RDFS.RDFSNS)

def make_label_query(uriBase, *uriTails):
    def labels(graph, lang, resource):
        labelBase = uriBase + "%s"
        class Labels(SlotStruct):
            __slots__ = uriTails
        return Labels(
                *[HasLabel(graph, lang, URIRef(labelBase % tail)).label
                        for tail in Labels.__slots__] )
    return labels

class Lang(RdfQuery):
    value = one(RDF.RDFNS)
    label = i18n_dict(RDFS.RDFSNS)




# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages
import oort

setup(
    name = "Oort",
    version = oort.__version__,
    description = """A WSGI-enabled toolkit for creating RDF-driven web apps.""",
    long_description = oort.__doc__,
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Paste",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules"
        ],
    keywords = "toolkit rdf web wsgi",
    author = "Niklas Lindström",
    author_email = "lindstream@gmail.com",
    license = "BSD",
    url = "http://oort.to/",
    #packages = find_packages(exclude=["*.test", "*.test.*", "test.*", "test"]),
    packages = find_packages(),
    include_package_data = True,
    zip_safe = False,
    test_suite = 'nose.collector',
    install_requires = ['rdflib >= 2.3',
                      'Paste >= 0.9',
                      'PasteScript >= 0.9',
                      'setuptools',
                      'Genshi >= 0.3.1'],
    entry_points="""
        [paste.paster_create_template]
        oort_app=oort.util.paste_templates:OortAppTemplate
        """
    )




# -*- coding: UTF-8 -*-
from oort.util._genshifilters import language_filtered_xml


XML = u"""
<span>XML.</span>
"""

XML_EN = u"""
<span xml:lang="en">Testing.</span>
"""

XML_SV = u"""
<span xml:lang="sv">Testar.</span>
"""

FULL_XML = u"""
<div xml:lang="en">Testing.</div>
<div xml:lang="sv">Testar.</div>
"""

WRAPPED = u"""<rdf-wrapper>
    <p><em>XML</em>.</p>
</rdf-wrapper>"""

WRAPPED_EN = u"""<rdf-wrapper xml:lang="en">
    <p>Some content.</p>
    <p>Some <em>more</em> content.</p>
</rdf-wrapper>"""

WRAPPED_SV = u"""<rdf-wrapper xml:lang="sv">
    <p>Lite inneåll.</p>
    <p>Lite <em>mer</em> innehåll.</p>
</rdf-wrapper>"""


def test_language_filtered_xml():
    for lang in 'en', 'sv':
        print "="*20, lang
        for xml in (XML, XML_EN, XML_SV, FULL_XML, WRAPPED, WRAPPED_EN, WRAPPED_SV):
            stream = language_filtered_xml(xml, lang)
            print stream.render()
    stream = language_filtered_xml([XML_EN, XML_SV], 'en')




# -*- coding: UTF-8 -*-
"""This module contains everything needed to setup an Oort-based web 
application. It also imports relevant parts from ``oort.display``.
"""
#=======================================================================
import urllib
import logging
from pkg_resources import EntryPoint
from rdflib import URIRef
from paste.wsgiwrappers import WSGIRequest, WSGIResponse
from paste.httpexceptions import HTTPTemporaryRedirect
from oort.display import DisplayBase, Display, SubTypeAwareDisplay
from oort.display import AspectBase, Aspect, TpltAspect, JsonAspect, RdfAspect
from oort.rdfview import RdfQuery
from oort.util.code import SlotStruct, contract, autosuper
__metaclass__ = autosuper
#=======================================================================
# FIXME: really, fix the "variant" concept..
# TODO: option to give ignored suffix for e.g. downloading..?
# TODO: Work on SubTypeAwareDisplay (partially done; consider using "poorMansInference")
#   - or <http://dev.w3.org/cvsweb/~checkout~/2004/PythonLib-IH/RDFSClosure.py>


class resource_viewer_meta(autosuper):
    def __init__(cls, clsName, bases, clsDict):
        autosuper.__init__(cls, clsName, bases, clsDict)
        displayClasses = clsDict.setdefault('displayClasses', {})
        defaultDisplay = clsDict.get('defaultDisplay')
        for member in clsDict.values():
            if isinstance(member, type) and issubclass(member, DisplayBase):
                name = getattr(member, 'name', None)
                if not name:
                    continue
                displayClasses[name] = member
                if not defaultDisplay and getattr(member, 'default', False):
                    defaultDisplay = name
        cls.displayClasses = displayClasses
        cls.defaultDisplay = defaultDisplay


class ResourceViewerBase:
    """Base class - not for direct usage."""
    __metaclass__ = resource_viewer_meta

    @classmethod
    def app_factory(cls, global_conf=None,
                    graph_factory=None, **cfg):
        """Factory method compatible with the app_factory feature of Paste Deploy."""
        mk_graph = EntryPoint.parse("x=%s" % graph_factory).load(False)
        graph = mk_graph()
        configure_logging(logging, cfg)
        return cls(graph, cfg)

    VARIANT_SEP = '-'
    RESOURCE_QUERY_KEY = 'resource'

    #defaultDisplay = None
    langOrder = ()

    def __init__(self, graph, cfg=None):
        self.graph = graph
        self._logger = logging.getLogger("oort.sitebase.ResourceViewerBase")
        self.displays = dict((name, dispCls(cfg=cfg))
                for name, dispCls in self.displayClasses.items())

    def __call__(self, environ, start_response):
        req = WSGIRequest(environ)

        self._viewPathTemplate = req.script_name + '/%s/%s/'
        args = req.path_info.split('/')
        try:
            args.pop(0) # remove first ''
            lang = args.pop(0)
            displayArg = args.pop(0)
        except IndexError:
            lang, displayArg = None, None
        if not displayArg:
            return self._default_redirect(lang, environ, start_response)

        uri = req.GET.get(self.RESOURCE_QUERY_KEY) or self.resource_from(args)

        get_result, contentType = self.prepared_display(
                URIRef(uri), lang, displayArg, req)

        headerMap = {}
        if contentType:
            headerMap['Content-Type'] = contentType

        start_response('200 OK', headerMap.items())
        return get_result(self.graph)

    def _default_redirect(self, lang, environ, start_response):
        redirect = HTTPTemporaryRedirect(
                self.app_url_for(lang or self.langOrder[0], self.defaultDisplay))
        return redirect(environ, start_response)

    @contract.template_method
    def resource_from(self, pathArgs):
        raise NotImplementedError

    @contract.template_method
    def resource_to_app_path(self, resource):
        raise NotImplementedError

    def app_url_for(self, lang, displayArg, resource=None, fallback=True):
        urlBase = self._viewPathTemplate % (lang, displayArg)
        if resource:
            resource, success = self.resource_to_app_path(resource)
            if success:
                return urlBase + resource.replace('#', '%23')
            elif fallback:
                return '%s?%s=%s' % (urlBase, self.RESOURCE_QUERY_KEY,
                                     urllib.quote(resource))
            else:
                return None # TODO: raise NoAppUrlForResourceError(resource)
        else:
            return urlBase

    def prepared_display(self, resource, lang, displayArg, req):
        args = displayArg.split(self.VARIANT_SEP, 1)
        name, variantArg = args[0], len(args) > 1 and args[1]
        display = self.displays[name]
        current = Current(resource, lang)
        appUtil = AppUtil(current, displayArg, self, req)

        self._logger.debug("Prepared display with: %s %s %s"
                            % (current, appUtil, display ))
        # TODO: use __repr__?

        def get_result(graph):
            return display.get_result_generator(graph, appUtil, variantArg)

        return get_result, display.contentType


LOG_LEVEL_NAMES = 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'

def configure_logging(logging, cfg):
    logCfg = {}
    level = cfg.get('logging_level', 'warning').upper()
    #format, filename, filemode
    if level in LOG_LEVEL_NAMES:
        logCfg['level'] = getattr(logging, level)
    logging.basicConfig(**logCfg)


class Current(SlotStruct):
    """Representation of the current *resource* request."""
    __slots__ = 'resource', 'lang'

class AppUtil(SlotStruct):
    """"An instance of this class is provided to the templates under the name 
    'app'. It provides a controlled interface to the application and the 
    current state (request)."""

    __slots__ = 'current', '_displayArg', '_viewer', '_req'

    def queries(self, queries):
        for query in queries:
            yield query(self._viewer.graph, self.current.resource, self.current.lang)

    def link(self, resource=None, lang=None, display=None, fallback=True, displayArg=None):
        """Returns an URL within the current app. The following arguments 
        can be given (all defaults to current unless provided):

            resource
                A target URI or an instance of an RdfQuery.

            lang
                The language to use.

            display
                The display name to use.

            fallback
                Revert to a GET for the resource if no app url can be made.
                Defaults to True.
        """
        if resource:
            if isinstance(resource, RdfQuery):
                resource = resource._subject
        else: resource = self.current.resource
        lang = lang or self.current.lang
        displayArg = display or displayArg or self._displayArg
        return self._viewer.app_url_for(lang, displayArg, resource, fallback)

    def safe_link(self, *args):
        return self.link(fallback=False)

    @property
    def baseurl(self):
        """Returns the base URL of this web app."""
        return self._req.script_name


class ResourceViewer(ResourceViewerBase):
    """Use for simple apps where all URIs in the graph share the same base URI 
    (set ``resourceBase`` to that)."""
    resourceBase =None
    trailSep='/'

    def __init__(self, *args):
        self.__super.__init__(*args)
        self._uriBase = self.resourceBase + "%s"

    def resource_from(self, pathArgs):
        return self._uriBase % self.trailSep.join(pathArgs)

    def resource_to_app_path(self, resource):
        base = self.resourceBase
        if resource.find(base) == 0:
            return str(resource)[len(base):], True
        else:
            return resource, False


class MultiBaseResourceViewer(ResourceViewerBase):
    """Use for full-fledged apps where many different base URIs will be 
    "mounted" at different named bases (done with the dictionary 
    ``resourceBases``)."""""
    resourceBases = {}
    defaultResource = ''
    # TODO: skip defaultResource and use defaultResourceBase?
    # TODO: if bad baseKey, raise Exception instead?

    def resource_from(self, pathArgs):
        if not pathArgs:
            return self.defaultResource
        pathArgs = list(pathArgs)
        baseKey = pathArgs.pop(0)
        lookup = self.resourceBases.get(baseKey)
        if not lookup:
            return self.defaultResource
        uriBase, trailSep = lookup
        return uriBase + trailSep.join(pathArgs)

    def resource_to_app_path(self, resource):
        for baseKey, (base, trailSep) in self.resourceBases.items():
            if resource.find(base) == 0:
                respath = str(resource)[len(base):]
                if trailSep:
                    respath = respath.replace(trailSep, '/')
                lpath = "%s/%s" % (baseKey, respath)
                return lpath, True
        return resource, False





# -*- coding: UTF-8 -*-
"""This module contains a system for creating rdf query classes in a mainly 
declarative manner. These are built by subclassing ``RdfQuery`` and creating 
class attributes by using ``Selector``:s. There are several selectors provided 
in this module to cover all regular cases of data acquisition.
"""
#=======================================================================
from rdflib import RDF, RDFS, Namespace, URIRef, BNode, Literal
from rdflib import ConjunctiveGraph
from types import ModuleType
try: import simplejson
except ImportError: simplejson = None
from oort.util.code import contract
#=======================================================================


THIS_QUERY = object()
DEFAULT_MAX_DEPTH = 20


# TODO: optimization by providing an "execution context" and avoiding multiple queries for the same uri etc.
# FIXME: complete; pass along execContext (and maxDepth?)
def run_queries(queries, graph, subject, lang):
    execContext = {}
    for query in queries:
        key = (id(query), str(subject))
        if key not in execContext:
            execContext[key] = query(graph, subject, lang)
        yield execContext[key]


#=======================================


class Selector(object):
    __slots__ = ('predicate', '_namespace', 'subQuery', 'filters',
                 '_name', '_queryClass')

    def __init__(self, predBase=None, subQuery=None):
        self.predicate = None
        self._namespace = None
        if isinstance(predBase, Namespace) or isinstance(predBase, ModuleType):
            self._namespace = predBase
        elif isinstance(predBase, URIRef):
            self.predicate = predBase
        self.subQuery = subQuery
        self.filters = []

    @contract.state_change
    def hook_into_rdf_query(self, name, queryClass):
        self._name = name
        self._queryClass = queryClass
        if self.subQuery is THIS_QUERY:
            self.subQuery = self._queryClass
        if not self._namespace:
            self._namespace = queryClass._namespace
        if not self.predicate and self._namespace:
            if isinstance(self._namespace, ModuleType):
                self.predicate = getattr(self._namespace, name)
            else:
                self.predicate = self._namespace[name]
        # TODO: not used since e.g. the selector decorator uses no predicate
        #if not self.predicate:
        #    raise ValueError(
        #            "Could not determine predicate for Selector %s" % self)

    def __get__(self, parentQuery, _parentQueryClass=None):
        prep = parentQuery._preparedSelects[self._name]
        if not prep.hasRun:
            result = self.retreive_result(parentQuery, prep.selectArgs)
            for fltr in self.filters:
                result = fltr(result)
            prep.hasRun = True
            prep.result = result
        return prep.result

    @contract.default_method
    def retreive_result(self, parentQuery, selectArgs):
        result = self._process_for_subqueries(
                self.select(*selectArgs),
                *selectArgs[:2]
            )
        return result

    # TODO: if subQuery is None but resource isn't a literal, use rdf:type to
    # find a proper RdfQuery (by looking in a register of RdfQuery subclasses
    # with a declared SCHEMA)? Or is type_switch enough?
    def _process_for_subqueries(self, rawresults, graph, lang):
        returnList = isinstance(rawresults, list)
        if not rawresults:
            if returnList: return []
            else: return None

        subQuery = self.subQuery
        if not subQuery:
            return rawresults
        else:
            # FIXME: using THIS_QUERY may currently cause infinite loops.
            if returnList:
                return [subQuery(graph, lang, uri) for uri in rawresults]
            else:
                return subQuery(graph, lang, rawresults)

    @contract.template_method
    def select(self, graph, lang, subject):
        raise NotImplementedError
        return None or []

    @contract.default_method
    def back_to_graph(self, graph, subject, value):
        pass

    def __rshift__(self, subQuery):
        self.subQuery = subQuery
        return self

    def __or__(self, fltr):
        self.filters.append(fltr)
        return self

    def __set__(self, parentQuery, value):
        prep = parentQuery._preparedSelects[self._name]
        lang = parentQuery._lang
        sub = self.subQuery
        if isinstance(value, list):
            value = [self.type_raw_value(val, lang) for val in value]
        else:
            value = self.type_raw_value(value, lang)
        if sub:
            if isinstance(value, list):
                prep.result = [sub.from_dict(val, lang, BNode())
                                for val in value]
            else:
                prep.result = sub.from_dict(value, lang, BNode())
        else:
            prep.result = value
            prep.hasRun = True

    @contract.default_method
    def type_raw_value(self, value, lang):
        # TODO: need more clever type mapping. also, allow {'_uri': ".." } to be resource?
        if isinstance(value, str.__base__):
            value = Literal(value)
        return value


class PreparedSelect(object):
    __slots__ = ('selectArgs', 'result', 'hasRun')
    def __init__(self, graph, lang, subject):
        self.selectArgs = (graph, lang, subject)
        self.result = None
        self.hasRun = False # TODO: make configurable?


class _rdf_query_meta(type):
    def __init__(cls, clsName, bases, clsDict):
        type.__init__(cls, clsName, bases, clsDict)

        cls._selectors = selectors = {}
        for base in bases:
            if hasattr(base, '_selectors'):
                selectors.update(base._selectors)

        rdfBase = clsDict.get('_rdfbase_')
        if not rdfBase:
            for base in bases:
                if hasattr(base, '_rdfbase_'):
                    rdfBase = base._rdfbase_
                    break
        if isinstance(rdfBase, Namespace):
            cls._namespace = rdfBase
            if not clsDict.get('RDF_TYPE'):
                cls.RDF_TYPE = rdfBase[clsName]
        else:
            cls._namespace = None # TODO: pick from type?

        for key, value in clsDict.items():
            if isinstance(value, Selector):
                value.hook_into_rdf_query(key, cls)
                selectors[key] = value


class RdfQuery(object):
    __metaclass__ = _rdf_query_meta

    RDF_TYPE = RDFS.Resource

    def __init__(self, graph, lang, subject, depth=DEFAULT_MAX_DEPTH):
        # TODO: pass depth and use to avoid infinite recursion
        if depth == 0:
            raise NotImplementedError("TODO: max depth exception..")

        self._graph = graph
        self._subject = subject
        self._lang = lang
        self._preparedSelects = prepareds = {}
        for name, selector in self._selectors.items():
            if not subject: # FIXME: happens when subject is a bnode?
                setattr(self, name, None)
                continue
            prep = PreparedSelect(graph, lang, subject)
            prepareds[name] = prep

    def __str__(self):
        return str(self._subject)

    def __eq__(self, other):
        if isinstance(other, RdfQuery):
            return self._subject == other._subject
        else:
            return self._subject == other

    @property
    def uri(self):
        return self._subject

    def get_selected_value(self, name):
        return self._preparedSelects[name].result

    @classmethod
    def bound_with(cls, subject, lang=None):
        def bound_query(graph, _lang, _subject):
            return cls(graph, lang or _lang, subject)
        bound_query.query = cls
        bound_query.__name__ = cls.__name__
        return bound_query

    def to_graph(self, lgraph=None):
        subject = self._subject or BNode() # FIXME: is this ok?
        if not subject: return # FIXME, see fixme in __init__

        lgraph = lgraph or ConjunctiveGraph()

        for t in self._graph.objects(subject, RDF.type):
            lgraph.add((subject, RDF.type, t))

        for selector in self._selectors.values():
            value = selector.__get__(self)
            if not value:
                continue
            selector.back_to_graph(lgraph, subject, value)

        # FIXME: why is this happening; how can we prevent it?
        for t in lgraph:
            if None in t: lgraph.remove(t)
        return lgraph

    def to_rdf(self):
        return self.to_graph().serialize(format='pretty-xml')

    def to_dict(self, keepSubject=False):
        d = {}
        if keepSubject:
            # TODO: sync with new property 'uri'
            subjectKey = isinstance(keepSubject, str) and keepSubject or 'resource'
            subj = self._subject
            if subj and not isinstance(subj, BNode):
                d[subjectKey] = self._subject

        for selector in self._selectors.values():
            name = selector._name
            value = selector.__get__(self)
            if not value:
                continue
            if isinstance(value, dict):
                d[name] = dict([(key, self.__dict_convert(val, keepSubject))
                            for key, val in value.items()])
            elif hasattr(value, '__iter__'):
                d[name] = [self.__dict_convert(val, keepSubject)
                            for val in value]
            else:
                d[name] = self.__dict_convert(value, keepSubject)
            # TODO: handle xml literals
        return d

    def __dict_convert(self, value, keepSubject):
        if isinstance(value, RdfQuery):
            return value.to_dict(keepSubject)
        else:
            return unicode(value) # TODO: simple type conversions?

    def to_json(self, keepSubject=False):
        if simplejson:
            return simplejson.dumps(self.to_dict(keepSubject))
        else:
            raise NotImplementedError

    @classmethod
    def from_dict(cls, data, lang, subject):
        graph = ConjunctiveGraph()
        query = cls(graph, lang, subject)
        for k, v in data.items():
            setattr(query, k, v)
        return query


class Filter(object):
    def __init__(self, func):
        self.func = func
    def __call__(self, items):
        return filter(self.func, items)


class Sorter(object):
    def __init__(self, attr=None, reverse=False, ignoreCase=False):
        self.attr = attr
        self.reverse = reverse
        self.ignoreCase = ignoreCase
    def __call__(self, items):
        copy = items[:]
        copy.sort(self.sort)
        if self.reverse:
            copy.reverse()
        return copy
    def sort(self, r1, r2):
        attr = self.attr
        if attr:
            v1, v2 = getattr(r1, attr), getattr(r2, attr)
        else:
            v1, v2 = r1, r2
        if self.ignoreCase:
            v1, v2 = v1.lower(), v2.lower()
        return cmp(v1, v2)


#=======================================


# TODO: totally untested!
#def type_switch(typeSelectors, default):
#    rdfType = graph.value(resource, RDF.type, None, any=True)
#    def select(graph, lang, resource):
#        return typeSelectors.get(rdfType, default)
#    return select



def back_from_value(graph, subject, predicate, value):
        if isinstance(value, RdfQuery):
            graph.add((subject, predicate, value._subject))
            value.to_graph(graph)
        else:
            if not isinstance(value, list): # TODO: fix this
                graph.add((subject, predicate, value))


class UnarySelector(Selector):
    def back_to_graph(self, graph, subject, value):
        back_from_value(graph, subject, self.predicate, value)

class EachSelector(Selector):
    def back_to_graph(self, graph, subject, values):
        for value in values:
            back_from_value(graph, subject, self.predicate, value)


class one(UnarySelector):
    def select(self, graph, lang, subject):
        return graph.value(subject, self.predicate, None, any=True)


class each(EachSelector):
    def select(self, graph, lang, subject):
        return list(graph.objects(subject, self.predicate))


class one_where_self_is(Selector):
    def select(self, graph, lang, subject):
        return graph.value(None, self.predicate, subject, any=True)

    def back_to_graph(self, graph, subject, value):
        back_from_value(graph, value._subject, self.predicate, subject)


class each_where_self_is(Selector):
    def select(self, graph, lang, subject):
        return list(graph.subjects(self.predicate, subject))

    def back_to_graph(self, graph, subject, values):
        for value in values:
            back_from_value(graph, value._subject, self.predicate, subject)


class collection(Selector):
    def __init__(self, base, subQuery=None, multiple=False):
        Selector.__init__(self, base, subQuery)
        self.multiple = multiple

    def select(self, graph, lang, subject):
        if self.multiple:
            allItems = [graph.items(res)
                        for res in graph.objects(subject, self.predicate)]
            return list(chain(*allItems))
        else:
            return list(graph.items(
                    graph.value(subject, self.predicate, None, any=True)
                ))

    def back_to_graph(self, graph, subject, values):
        if not values:
            graph.add((subject, self.predicate, RDF.nil))
            return
        bnode = BNode()
        graph.add((subject, self.predicate, bnode))
        for value in values:
            back_from_value(graph, bnode, RDF.first, value)
            newBnode = BNode()
            graph.add((bnode, RDF.rest, newBnode))
            bnode = newBnode
        graph.add((bnode, RDF.rest, RDF.nil))


class TypeLocalized(Selector):
    def type_raw_value(self, value, lang):
        if isinstance(value, str.__base__):
            value = Literal(value, lang)
        return value

class localized(TypeLocalized, UnarySelector):
    def select(self, graph, lang, subject):
        first = None
        for value in graph.objects(subject, self.predicate):
            if not first: first = value
            if getattr(value, 'language', None) == lang:
                return value
        return first


# TODO: This is a hackish solution; see also below (transparently using datatype).
#       It also reduces the literal, making it irreversible.
from oort.util._genshifilters import language_filtered_xml
class localized_xml(UnarySelector):
    """This selector removes any elements with an xml:lang other than the 
    current language. It also supports the never standardized 'rdf-wrapper' in 
    XML Literals, who are removed from the output.

    Important! This is currently tied to the Genshi Templating System, and may 
    not work as expected in all cases."""
    def select(self, graph, lang, subject):
        return language_filtered_xml(
                graph.objects(subject, self.predicate), lang)

    def type_raw_value(self, value, lang):
        if isinstance(value, str.__base__):
            value = Literal(value, datatype=RDF.XMLLiteral)
        return value

class i18n_dict(Selector):
    def select(self, graph, lang, subject):
        valueDict = {}
        for value in graph.objects(subject, self.predicate):
            valueDict[value.language] = value
        return valueDict

    def back_to_graph(self, graph, subject, value):
        for lang, text in value.items():
            graph.add((subject, self.predicate, Literal(text, lang=lang)))


class each_localized(TypeLocalized, EachSelector):
    def select(self, graph, lang, subject):
        return [ value for value in graph.objects(subject, self.predicate)
                 if value.language == lang ]


#=======================================


class selector(Selector):
    "Use as decorator for methods of an RdfQuery subclass to convert them to selectors."
    def __init__(self, func):
        super(selector, self).__init__(None)
        self.func = func
    def retreive_result(self, parentQuery, selectArgs):
        return self.func(parentQuery, *selectArgs)


#=======================================
# TODO: consider returning ElementTree data for XML Literals. And if so, also
# filtered on u'{http://www.w3.org/XML/1998/namespace}lang' (keep if none) for localized.

# TODO: also consider checking datatype and coercing (at least) these:
# Use: rdflib.Literal.castPythonToLiteral
# See: rdflib.sparql.sparqlOperators.getLiteralValue(v)
# See: <http://en.wikipedia.org/wiki/RDFLib#RDF_Literal_Support>


#=======================================

def chain(*iters):
    for iterator in iters:
        for i in iterator: yield i


